# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.0.0

import winrt.system
from winrt._winrt_windows_devices_background import (
    DeviceServicingDetails,
    DeviceUseDetails,
)

__all__ = [
    "DeviceServicingDetails",
    "DeviceUseDetails",
]

